
#    MIDAS Httpd Server to provide Data Acquisition Control Services

global env
source [file join $env(MIDASBASE) TclHttpd tcl MIDASDomainHandler.tcl]       ;#  load default support procedures

global MIDAS_DataAcquisitionControl_Verbose
set MIDAS_DataAcquisitionControl_Verbose 0

global MIDAS_SOAPDomainHandler_Verbose
set MIDAS_SOAPDomainHandler_Verbose 0


global MIDAS_DataAcquisitionCode MIDAS_DataAcquisitionState MIDAS_DataAcquisitionReason
set MIDAS_DataAcquisitionCode 2

#    state returns

#    code (int)  state (string)  reason  (string)

#       0           stopped
#       1           going
#       2           initialised    "please setup"
#       3           setting up
#       4           changing to going
#       5           changing to stopped
#       6           error           ******
#       7           undefined       ******

set MIDAS_DataAcquisitionState(0) "stopped"
set MIDAS_DataAcquisitionState(1) "going"
set MIDAS_DataAcquisitionState(2) "initialised"
set MIDAS_DataAcquisitionState(3) "setting up"
set MIDAS_DataAcquisitionState(4) "changing to going"
set MIDAS_DataAcquisitionState(5) "changing to stopped"
set MIDAS_DataAcquisitionState(6) "error"
set MIDAS_DataAcquisitionState(7) "undefined"

set MIDAS_DataAcquisitionReason(0) ""
set MIDAS_DataAcquisitionReason(1) ""
set MIDAS_DataAcquisitionReason(2) "please setup"
set MIDAS_DataAcquisitionReason(3) "please wait"
set MIDAS_DataAcquisitionReason(4) "please wait"
set MIDAS_DataAcquisitionReason(5) "please wait"
set MIDAS_DataAcquisitionReason(6) "try setup"
set MIDAS_DataAcquisitionReason(7) ""


# Use namespaces to isolate our methods
namespace eval urn:DataAcquisitionControl {

    proc ServerVerbose {opt} {
        do_ServerVerbose $opt
        return [rpcvar::rpcvar int 0]
    }

    proc Get-State {args} {
        set vl [do_Get-State]
        return [list [list Code int [first $vl]]  [list State string [second $vl]]  [list Reason string [third $vl]]]
    }

    proc Do-SetUp {args} {
        set vl [do-SetUp]
        return [rpcvar::rpcvar int $vl]
    }

    proc Do-Stop {args} {
        set vl [do-Stop]
        return [rpcvar::rpcvar int $vl]
    }

    proc Do-Go {args} {
        set vl [do-Go]
        return [rpcvar::rpcvar int $vl]
    }

    proc Get-Counters {args} {
        set vl [do_Get-Counters]
        return [list [list GoodEvents int [lindex $vl 0]] [list BadEvents int [lindex $vl 1]] [list InputBuffers int [lindex $vl 2]] [list OutputBuffers int [lindex $vl 3]]]
    }

    proc Get-Rates {args} {
        set vl [do_Get-Rates]
        return [list [list GoodEvents int [lindex $vl 0]] [list BadEvents int [lindex $vl 1]] [list InputBuffers int [lindex $vl 2]] [list OutputBuffers int [lindex $vl 3]]]
    }

    proc do_ServerVerbose {option} {
        global MIDAS_DataAcquisitionControl_Verbose
        if {$MIDAS_DataAcquisitionControl_Verbose} {puts "called method ServerVerbose with parameter $option"}
        set MIDAS_DataAcquisitionControl_Verbose $option
    }

    proc do_Get-State {} {
       global env

       set env(EGsilent) 1
       set z [catch {inform sys Get_State} m]
       set env(EGsilent) 0

       if {$z == 0} then {
          return $m
       } else {
           return -code error -errorcode 0x10005 -errorinfo "received error response: rc=$z\n  $m"
       }
    }

    proc do-SetUp {} {
       global env

       set env(EGsilent) 1
       set z [catch {inform sys do_Setup} m]
       set env(EGsilent) 0

       if {$z == 0} then {
          return $m
       } else {
           return -code error -errorcode 0x10005 -errorinfo "received error response: rc=$z\n  $m"
       }
    }

    proc do-Stop {} {
       global env

       set env(EGsilent) 1
       set z [catch {inform sys do_Stop} m]
       set env(EGsilent) 0

       if {$z == 0} then {
          return $m
       } else {
           return -code error -errorcode 0x10005 -errorinfo "received error response: rc=$z\n  $m"
       }
    }

    proc do-Go {} {
        global env

       set env(EGsilent) 1
       set z [catch {inform sys do_Go} m]
       set env(EGsilent) 0

       if {$z == 0} then {
          return $m
       } else {
           return -code error -errorcode 0x10005 -errorinfo "received error response: rc=$z\n  $m"
       }
    }

    proc do_Get-Counters {} {
        global env

       set env(EGsilent) 1
       set z [catch {inform stats Get_Counters} m]
       set env(EGsilent) 0

       if {$z == 0} then {
          return $m
       } else {
           return -code error -errorcode 0x10005 -errorinfo "received error response: rc=$z\n  $m"
       }
    }

    proc do_Get-Rates {} {
       global env

       set env(EGsilent) 1
       set z [catch {inform stats Get_Rates} m]
       set env(EGsilent) 0

       if {$z == 0} then {
          return $m
       } else {
           return -code error -errorcode 0x10005 -errorinfo "received error response: rc=$z\n  $m"
       }
    }



    # We have to publish the public methods...

     SOAP::export ServerVerbose
     SOAP::export Get-State
     SOAP::export Do-SetUp
     SOAP::export Do-Stop
     SOAP::export Do-Go
     SOAP::export Get-Counters
     SOAP::export Get-Rates
}

# register this service with tclhttpd
puts [SOAP::Domain::register  -prefix  /DataAcquisitionControl -namespace urn:DataAcquisitionControl -uri urn:DataAcquisitionControl]


proc ::SOAP::Domain::MIDAS_SOAPDomainHandler_/DataAcquisitionControl {optsname sock args} {   \
    ::SOAP::Domain::MIDAS_SOAPDomainHandler_/DefaultAccessServer $optsname $sock $args}
