
#    MIDAS Httpd Server to provide Data Acquisition Control Services

global env
source [file join $env(MIDASBASE) TclHttpd tcl MIDASDomainHandler.tcl]       ;#  load default support procedures

global MIDAS_DataAcquisitionControl_Verbose
set MIDAS_DataAcquisitionControl_Verbose 1


 global MIDAS_SOAPDomainHandler_Verbose
 set MIDAS_SOAPDomainHandler_Verbose 1


global MIDAS_DataAcquisitionCode MIDAS_DataAcquisitionState MIDAS_DataAcquisitionReason
set MIDAS_DataAcquisitionCode 2

#    state returns

#    code (int)  state (string)  reason  (string)

#       0           stopped
#       1           going
#       2           initialised    "please setup"
#       3           setting up
#       4           changing to going
#       5           changing to stopped
#       6           error           ******
#       7           undefined       ******

set MIDAS_DataAcquisitionState(0) "stopped"
set MIDAS_DataAcquisitionState(1) "going"
set MIDAS_DataAcquisitionState(2) "initialised"
set MIDAS_DataAcquisitionState(3) "setting up"
set MIDAS_DataAcquisitionState(4) "changing to going"
set MIDAS_DataAcquisitionState(5) "changing to stopped"
set MIDAS_DataAcquisitionState(6) "error"
set MIDAS_DataAcquisitionState(7) "undefined"

set MIDAS_DataAcquisitionReason(0) ""
set MIDAS_DataAcquisitionReason(1) ""
set MIDAS_DataAcquisitionReason(2) "please setup"
set MIDAS_DataAcquisitionReason(3) "please wait"
set MIDAS_DataAcquisitionReason(4) "please wait"
set MIDAS_DataAcquisitionReason(5) "please wait"
set MIDAS_DataAcquisitionReason(6) "try setup"
set MIDAS_DataAcquisitionReason(7) ""

global MIDAS_DataAcquisitionCounters MIDAS_DataAcquisitionRates

#    the order of counters and rates returned is

#    1  good events   (counter 0)
#    2  bad events    (counter 11)
#    3  input data buffers  (for non buffered input return 0)
#    4  output data buffers  (counter 4)

set MIDAS_DataAcquisitionCounters(0) 0
set MIDAS_DataAcquisitionCounters(1) 0
set MIDAS_DataAcquisitionCounters(2) 0
set MIDAS_DataAcquisitionCounters(3) 0

set MIDAS_DataAcquisitionRates(0) 0
set MIDAS_DataAcquisitionRates(1) 0
set MIDAS_DataAcquisitionRates(2) 0
set MIDAS_DataAcquisitionRates(3) 0


# Use namespaces to isolate our methods
namespace eval urn:DataAcquisitionControl {

    proc ServerVerbose {opt} {
        do_ServerVerbose $opt
        return [rpcvar::rpcvar int 0]
    }

    proc Get-State {args} {
        set vl [do_Get-State]
        return [list [list Code int [first $vl]]  [list State string [second $vl]]  [list Reason string [third $vl]]]
    }

    proc Do-SetUp {args} {
        set vl [do-SetUp]
        return [rpcvar::rpcvar int $vl]
    }

    proc Do-Stop {args} {
        set vl [do-Stop]
        return [rpcvar::rpcvar int $vl]
    }

    proc Do-Go {args} {
        set vl [do-Go]
        return [rpcvar::rpcvar int $vl]
    }

    proc Get-Counters {args} {
        set vl [do_Get-Counters]
        return [list [list GoodEvents int [lindex $vl 0]] [list BadEvents int [lindex $vl 1]] [list InputBuffers int [lindex $vl 2]] [list OutputBuffers int [lindex $vl 3]]]
    }

    proc Get-Rates {args} {
        set vl [do_Get-Rates]
        return [list [list GoodEvents int [lindex $vl 0]] [list BadEvents int [lindex $vl 1]] [list InputBuffers int [lindex $vl 2]] [list OutputBuffers int [lindex $vl 3]]]
    }

    proc do_ServerVerbose {option} {
        global MIDAS_DataAcquisitionControl_Verbose
        if {$MIDAS_DataAcquisitionControl_Verbose} {puts "called method ServerVerbose with parameter $option"}
        set MIDAS_DataAcquisitionControl_Verbose $option
    }

    proc do_Get-State {} {
       global MIDAS_DataAcquisitionCode MIDAS_DataAcquisitionState MIDAS_DataAcquisitionReason

       return [list $MIDAS_DataAcquisitionCode [set MIDAS_DataAcquisitionState($MIDAS_DataAcquisitionCode)] [set MIDAS_DataAcquisitionReason($MIDAS_DataAcquisitionCode)]]
    }

    proc do-SetUp {} {
       global MIDAS_DataAcquisitionCode MIDAS_DataAcquisitionState MIDAS_DataAcquisitionReason

       if {$MIDAS_DataAcquisitionCode == 0 || $MIDAS_DataAcquisitionCode == 2 || $MIDAS_DataAcquisitionCode == 6} {
           set MIDAS_DataAcquisitionCode 3
           after 10000 "set MIDAS_DataAcquisitionCode 0"
           set rc 0
       } else {
#           return -code error -errorcode 0x10001 -errorinfo "currently in wrong state for requested action"
           set rc 1
       }
       return $rc
    }

    proc do-Stop {} {
       global MIDAS_DataAcquisitionCode MIDAS_DataAcquisitionState MIDAS_DataAcquisitionReason

       if {$MIDAS_DataAcquisitionCode == 1 || $MIDAS_DataAcquisitionCode == 2 || $MIDAS_DataAcquisitionCode == 6} {
           set MIDAS_DataAcquisitionCode 5
           after 3000 "set MIDAS_DataAcquisitionCode 0"
           set rc 0
       } else {
#           return -code error -errorcode 0x10001 -errorinfo "currently in wrong state for requested action"
           set rc 1
       }
       return $rc
    }

    proc do-Go {} {
       global MIDAS_DataAcquisitionCode MIDAS_DataAcquisitionState MIDAS_DataAcquisitionReason

       if {$MIDAS_DataAcquisitionCode == 0} {
           set MIDAS_DataAcquisitionCode 4
           after 5000 "set MIDAS_DataAcquisitionCode 1"
           set rc 0
       } else {
#           return -code error -errorcode 0x10001 -errorinfo "currently in wrong state for requested action"
           set rc 1
       }
       return $rc
    }

    proc do_Get-Counters {} {
       global MIDAS_DataAcquisitionCounters

       incr MIDAS_DataAcquisitionCounters(0)
       incr MIDAS_DataAcquisitionCounters(1)
       incr MIDAS_DataAcquisitionCounters(2)
       incr MIDAS_DataAcquisitionCounters(3)

       return [list $MIDAS_DataAcquisitionCounters(0) $MIDAS_DataAcquisitionCounters(1) $MIDAS_DataAcquisitionCounters(2) $MIDAS_DataAcquisitionCounters(3)]
    }

    proc do_Get-Rates {} {
       global MIDAS_DataAcquisitionRates

       set MIDAS_DataAcquisitionRates(0) [expr ($MIDAS_DataAcquisitionRates(0) + 1) & 3]
       set MIDAS_DataAcquisitionRates(1) [expr ($MIDAS_DataAcquisitionRates(1) + 1) & 7]
       set MIDAS_DataAcquisitionRates(2) [expr ($MIDAS_DataAcquisitionRates(2) + 1) & 15]
       set MIDAS_DataAcquisitionRates(3) [expr ($MIDAS_DataAcquisitionRates(3) + 1) & 31]

       return [list $MIDAS_DataAcquisitionRates(0) $MIDAS_DataAcquisitionRates(1) $MIDAS_DataAcquisitionRates(2) $MIDAS_DataAcquisitionRates(3)]
    }



    # We have to publish the public methods...

     SOAP::export ServerVerbose
     SOAP::export Get-State
     SOAP::export Do-SetUp
     SOAP::export Do-Stop
     SOAP::export Do-Go
     SOAP::export Get-Counters
     SOAP::export Get-Rates
}

# register this service with tclhttpd
puts [SOAP::Domain::register  -prefix  /DataAcquisitionControl -namespace urn:DataAcquisitionControl -uri urn:DataAcquisitionControl]

proc ::SOAP::Domain::MIDAS_SOAPDomainHandler_/DataAcquisitionControl {optsname sock args} {   \
    ::SOAP::Domain::MIDAS_SOAPDomainHandler_/DefaultAccessServer $optsname $sock $args}
